library(ape)

humanTopHitNum <- NULL; 
tetrapodsNum   <- NULL; teleostsNum <- NULL
orthoCladeNum  <- NULL; 
reArrNum       <- NULL
teleostNodeNum <- NULL; 

infileName     <- "infile.txt"
outfileName    <- "treePlot.png"

teleostsPreFixes  <- c('Zebrafish_','Medaka_','Stickleback_','Tetraodon_','Fugu_')
tetrapodsPreFixes <- c('Chicken_','Xenopus_','Human_')

infile <- read.table(infileName, na.strings = FALSE)


linePickUpFromModFasta <- function(lines, keyWord)
{
  container <- c()
  frag <- 0
  lineStock <- c()
  for(line in lines) {
#    print(line)

    ### Collect lines 
   if (regexpr('^>', line) < 0) {
     if(frag == 1) {
       lineStock <- c(lineStock, line)
     }
   }
  
   if (regexpr('^>', line) > 0)
   {
      if(frag == 1)
      {
        container <- c(container, lineStock)
        lineStock <- c()
        break
      }

      keyWord1 <- paste('>', keyWord, sep='')
      if (regexpr(keyWord1, line) > 0)
      {
        container <- c(container, line)
        frag <- 1
      }
    }
  }
  container <- c(container, lineStock)    
  container <- container[-1]
  #print('container')
  #print(container)
#  cat('\n')
  return(container)
}


## A function to check the presence or absence of subject
presenceCheckInModFasta <- function (linesA, keyWordA)
{
  if(any(i <- grep(keyWordA,infile$V1)))
  {
    containerA <- linePickUpFromModFasta(linesA,keyWordA)
  } else {
    # print (paste('  ',keyWordA,' does not exist.', sep=''))
    containerA  <- NULL
  }
  return(containerA)
}

fishLeaves     <- presenceCheckInModFasta(infile$V1, "Teleosts") 
OrthoLeaves    <- presenceCheckInModFasta(infile$V1, "OrthoLeaves") 
humanTopHit    <- presenceCheckInModFasta(infile$V1, "humanTopHit") 

tr             <- presenceCheckInModFasta(infile$V1, "newickTree") 
tr <- read.tree(text = tr)
tr <- ladderize(tr, FALSE) # ladderize(tr,FALSE)


########## Main program
## Pick up the tip numbers
for(i in 1:length(tr$tip.label)){

  ## human query
  if(regexpr(humanTopHit, tr$tip.label[i]) > 0){
    humanTopHitNum <- i
  }

  ## tetrapods
  for (tetPF in tetrapodsPreFixes){
    if(regexpr(tetPF, tr$tip.label[i]) > 0){
      tetrapodsNum <- c(tetrapodsNum,i)
    }
  }

  for (teleostPF in teleostsPreFixes){
    if(regexpr(teleostPF,tr$tip.label[i]) > 0){
      teleostsNum <- c(teleostsNum,i)
    }
  }

  for(OrthoLeaf in OrthoLeaves){
    if(tr$tip.label[i] == OrthoLeaf){
      orthoCladeNum <- c(orthoCladeNum,i)
    }
  }

  ## fish node
  for(fishLeaf in fishLeaves){
    if(tr$tip.label[i] == fishLeaf){
      teleostNodeNum <- c(teleostNodeNum,i)
    }
  }

}


########## Get the number list
## Pick up the node numbers
for(p in 1:length(tr$node.label)){
  if(regexpr("^r", tr$node.label[p]) > 0){
    reArrNum <- c(reArrNum,p)
  }
}

## Bold character
fontNum                 <- rep(1, length(tr$tip.label))
fontNum[humanTopHitNum] <- 4

fontColors               <- rep("black", length(tr$tip.label))
fontColors[teleostsNum]  <- "blue"
fontColors[tetrapodsNum] <- "red"

nodeLabelColors           <- rep("black", length(tr$tip.label))
nodeLabelColors[reArrNum] <- "red"

nodeLabelFonts           <- rep(1, length(tr$tip.label))
nodeLabelFonts[reArrNum] <- 2

nodeLabelFontSize           <- rep(0.9, length(tr$tip.label))
nodeLabelFontSize[reArrNum] <- 1.2

## Tip label color
#teleosts1TipLabelColor        　      <- rep("white", length(tr$tip.label))
teleosts1TipLabelColor                 <- rep(0      , length(tr$tip.label))
teleosts1TipLabelColor[teleostNodeNum] <- 4

teleosts1TipLabelPCH                 <- rep(1,length(tr$tip.label))
teleosts1TipLabelPCH[teleostNodeNum] <- 1

## Branch width
if(is.null(orthoCladeNum)){
  edgeWidth       <- rep(1.5, dim(tr$edge)[1])
} else {
  whHF            <- which.edge(tr, orthoCladeNum)
  edgeWidth       <- rep(1.5, dim(tr$edge)[1])
  edgeWidth[whHF] <- 4
}


## Printout
png(outfileName) 
plot(tr, no.margin=TRUE, underscore = TRUE, use.edge.length=FALSE, label.offset = 3, cex = 0.9, font = fontNum, tip.col = fontColors, edge.width = edgeWidth)

nodelabels(tr$node.label, adj = c(1.2,-0.5), cex=nodeLabelFontSize, frame = "n", col = nodeLabelColors, font = nodeLabelFonts)

tiplabels(pch = teleosts1TipLabelPCH, col = teleosts1TipLabelColor, adj = 2.5, cex = 1.5)

dev.off() 
